/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <gpdef.h>
#include <gpstdlib.h>
#include <gpgraphic.h>
#include <gpstdio.h>
#include <gpfont.h>
#include <stdlib.h>

extern GPDRAWSURFACE screen[2];

void filer_init(void) {
	GpFatInit();
	GpRelativePathSet("gp:\\gpmm\\dragon");
}

int filer_open(char *filename, int mode, FILER_FD *fd) {
	if (mode & FILER_WRITE) {
		if (GpFileCreate(filename, ALWAYS_CREATE, fd) != SM_OK)
			return -1;
	} else {
		if (GpFileOpen(filename, OPEN_R, fd) != SM_OK)
			return -1;
	}
	return 0;
}

int filer_read(FILER_FD fd, void *buffer, Uint32 size) {
	unsigned long count;
	int rbytes = 0;
	ERR_CODE err;
	do {
		err = GpFileRead(fd, buffer, size>512?512:size, &count);
		if (err != SM_OK && err != ERR_EOF)
			return -1;
		size -= count;
		buffer += count;
		rbytes += count;
	} while (err == SM_OK && size > 0);
	return rbytes;
}

int filer_write(FILER_FD fd, void *buffer, Uint32 size) {
	ERR_CODE err;
	err = GpFileWrite(fd, buffer, (unsigned long)size);
	if (err != SM_OK)
		return -1;
	return size;
}

void filer_close(FILER_FD fd) {
	GpFileClose(fd);
}

FILER_DIRENT *filer_directory = NULL;
Uint32 filer_dir_count;

static int mstrncmp(char *s1, char *s2, Uint32 n) {
	for (; n; n--) {
		if (*s1 < *s2) return -1;
		if (*s1 > *s2) return 1;
		if (!(*s1)) return 0;
		s1++; s2++;
	}
	return 0;
}

static int dirsort(const void *a, const void *b) {
	FILER_DIRENT *aa = (FILER_DIRENT *)a;
	FILER_DIRENT *bb = (FILER_DIRENT *)b;
	if ((aa->attr.attr & 0x10) != (bb->attr.attr & 0x10))
		return (aa->attr.attr&0x10) ? -1 : 1;
	return mstrncmp(aa->name, bb->name, 16);
}

void filer_readdir(char *path) {
	GPDIRENTRY entry;
	unsigned long p_num, dummy;
	int i;
	if (filer_directory != NULL)
		free(filer_directory);
	GpRelativePathSet(path);
	GpDirEnumNum(".", &p_num);
	p_num -= 2;
	filer_directory = (FILER_DIRENT *)malloc(p_num * sizeof(FILER_DIRENT));
	for (i = 0; i < p_num; i++) {
		GpDirEnumList(".", i+2, 1, &entry, &dummy);
		memcpy(filer_directory[i].name, entry.name, 16);
		GpFileAttr(filer_directory[i].name, &filer_directory[i].attr);
		if (!mstrncmp(filer_directory[i].name, "..", 3))
			filer_directory[i].attr.attr |= 0x10;
	}
	qsort(filer_directory, p_num, sizeof(FILER_DIRENT), dirsort);
	filer_dir_count = p_num;
}
