#include <stdio.h>
#include <stdlib.h>
#include <SDL.h>
#include <SDL_image.h>

#ifdef BPP_16
#define MAPCOLOUR(r,g,b) ((r & 0xf8) << 8 | (g & 0xf8) << 3 | (b & 0xf8) >> 2)
#endif

#ifdef BPP_8
#define MAPCOLOUR(r,g,b) ((r & 0xc0) | (g & 0xe0) >> 2 | (b & 0xe0) >> 5)
#endif

Uint32 getpixel(SDL_Surface *surface, int x, int y);

int main(int argc, char **argv) {
	Uint32 p;
	Uint8 r,g,b;
	int x, y, xres, yres;
	SDL_Surface *in;
	int multiple, ino;
	if (argc < 3) {
		printf("Usage: %s name infile ...\n", argv[0]);
		exit(1);
	}
	multiple = (argc > 3);
	in = IMG_Load(argv[2]);
	xres = in->w; yres = in->h;
	printf("#ifdef GP32\n#include \"gp32/gptypes.h\"\n#else\n#include <SDL/SDL_types.h>\n#endif\n\n#include \"video.h\"\n\n");
	if (multiple)
		printf("PIXEL %s[%d][%d] = {\n{\n", argv[1], argc-2, xres*yres);
	else
		printf("PIXEL %s[%d] = {\n", argv[1], xres*yres);
	for (ino = 3; ino <= argc; ino++) {
		for (x = 0; x < xres; x++) {
			printf("\t");
			for (y = yres - 1; y >= 0; y--) {
				SDL_LockSurface(in);
				p = getpixel(in, x, y);
				SDL_UnlockSurface(in);
				SDL_GetRGB(getpixel(in, x, y), in->format, &r, &g, &b);
#ifdef BPP_16
				printf("0x%04x, ", MAPCOLOUR(r,g,b));
#else
				printf("0x%02x, ", MAPCOLOUR(r,g,b));
#endif
			}
			printf("\n");
		}
		SDL_FreeSurface(in);
		printf("}");
		if (ino < argc) {
			printf(",\n{\n");
			in = IMG_Load(argv[ino]);
		}
	}
	if (multiple)
		printf("\n}");
	printf(";\n");
	return 0;
}

Uint32 getpixel(SDL_Surface *surface, int x, int y) {
	int bpp = surface->format->BytesPerPixel;
	Uint8 *p = (Uint8 *)surface->pixels + y * surface->pitch + x * bpp;

	switch(bpp) {
		case 1: return *p;
		case 2: return *(Uint16 *)p;
		case 3:
			if (SDL_BYTEORDER == SDL_BIG_ENDIAN)
				return p[0] << 16 | p[1] << 8 | p[2];
			else
				return p[0] | p[1] << 8 | p[2] << 16;
		case 4: return *(Uint32 *)p;
		default: return 0;
	}
}
