/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>
#include <SDL/SDL_types.h>

void filer_init(void) {
}

int filer_open(char *filename, int mode, FILER_FD *fd) {
	if (mode & FILER_WRITE)
		*fd = open(filename, O_CREAT|O_WRONLY, 0644);
	else
		*fd = open(filename, O_RDONLY);
	if (*fd == -1) {
		filer_error = strerror(errno);
		return -1;
	}
	return 0;
}

int filer_read(FILER_FD fd, void *buffer, Uint32 size) {
	Uint32 count = 0, tries = 3;
	int r;
	do {
		do {
			r = read(fd, buffer, size);
		} while (r == -1 && errno == EINTR && --tries);
		if (r == -1) {
			filer_error = strerror(errno);
			return -1;
		}
		size -= r;
		buffer += r;
		count += r;
	} while (r && size > 0);
	return count;
}

int filer_write(FILER_FD fd, void *buffer, Uint32 size) {
	Uint32 count = 0, tries = 3;
	int r;
	do {
		do {
			r = write(fd, buffer, size);
		} while (r == -1 && errno == EINTR && --tries);
		if (r == -1) {
			filer_error = strerror(errno);
			return -1;
		}
		size -= r;
		buffer += r;
		count += r;
	} while (r && size > 0);
	return count;
}

void filer_close(FILER_FD fd) {
	close(fd);
}
