/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef GP32
#include "gpdef.h"
#include "gpstdlib.h"
#include "gpgraphic.h"
#include "gpstdio.h"
#include "gpfont.h"
#include "gp32/gptypes.h"
#else
#include <SDL/SDL_types.h>
#endif

#include "xroar.h"
#include "m6809.h"
#include "sam.h"
#include "vdg.h"
#include "pia.h"
#include "snapshot.h"
#include "logging.h"
#include "filer.h"
#include "filepick.h"

/* Pretty lousy snapshot code - just to assist in testing.  Doesn't
 * store SAM or PIA states. */

int snapshot_state = 0;

static char *get_filename(void) {
	//return "snap.sna";
	return filepick();
}

int write_snapshot(void) {
	FILER_FD fd;
	char *filename = get_filename();
	if (filer_open(filename, FILER_WRITE, &fd) == -1)
		return -1;
	filer_write_byte(fd, reg_cc);
	filer_write_byte(fd, reg_a);
	filer_write_byte(fd, reg_b);
	filer_write_byte(fd, reg_dp);
	filer_write_word16(fd, reg_x);
	filer_write_word16(fd, reg_y);
	filer_write_word16(fd, reg_u);
	filer_write_word16(fd, reg_s);
	filer_write_word16(fd, reg_pc);
	filer_write(fd, ram0, sizeof(ram0));
	/* PIA_0A */
	filer_write_byte(fd, PIA_0A.direction_register);
	filer_write_byte(fd, PIA_0A.output_register);
	filer_write_byte(fd, PIA_0A.control_register);
	/* PIA_0B */
	filer_write_byte(fd, PIA_0B.direction_register);
	filer_write_byte(fd, PIA_0B.output_register);
	filer_write_byte(fd, PIA_0B.control_register);
	/* PIA_1A */
	filer_write_byte(fd, PIA_1A.direction_register);
	filer_write_byte(fd, PIA_1A.output_register);
	filer_write_byte(fd, PIA_1A.control_register);
	/* PIA_1B */
	filer_write_byte(fd, PIA_1B.direction_register);
	filer_write_byte(fd, PIA_1B.output_register);
	filer_write_byte(fd, PIA_1B.control_register);
	/* SAM */
	filer_write_word16(fd, sam_register);
	return 0;
}

int read_snapshot(void) {
	FILER_FD fd;
	Uint8 tmp8;
	Uint16 tmp16;
	char *filename = get_filename();
	if (filename == NULL)
		return -1;
	xroar_reset();
	if (filer_open(filename, FILER_READ, &fd) == -1)
		return -1;
	filer_read_byte(fd, &reg_cc);
	filer_read_byte(fd, &reg_a);
	filer_read_byte(fd, &reg_b);
	filer_read_byte(fd, &reg_dp);
	filer_read_word16(fd, &reg_x);
	filer_read_word16(fd, &reg_y);
	filer_read_word16(fd, &reg_u);
	filer_read_word16(fd, &reg_s);
	filer_read_word16(fd, &reg_pc);
	filer_read(fd, ram0, sizeof(ram0));
	/* PIA_0A */
	filer_read_byte(fd, &tmp8);
	PIA_0A.register_select = 0; pia_data_write(&PIA_0A, tmp8);  /* DDR */
	filer_read_byte(fd, &tmp8);
	PIA_0A.register_select = 1; pia_data_write(&PIA_0A, tmp8);  /* OR */
	filer_read_byte(fd, &tmp8);
	pia_control_write(&PIA_0A, tmp8);  /* CR */
	/* PIA_0B */
	filer_read_byte(fd, &tmp8);
	PIA_0B.register_select = 0; pia_data_write(&PIA_0B, tmp8);  /* DDR */
	filer_read_byte(fd, &tmp8);
	PIA_0B.register_select = 1; pia_data_write(&PIA_0B, tmp8);  /* OR */
	filer_read_byte(fd, &tmp8);
	pia_control_write(&PIA_0B, tmp8);  /* CR */
	/* PIA_1A */
	filer_read_byte(fd, &tmp8);
	PIA_1A.register_select = 0; pia_data_write(&PIA_1A, tmp8);  /* DDR */
	filer_read_byte(fd, &tmp8);
	PIA_1A.register_select = 1; pia_data_write(&PIA_1A, tmp8);  /* OR */
	filer_read_byte(fd, &tmp8);
	pia_control_write(&PIA_1A, tmp8);  /* CR */
	/* PIA_1B */
	filer_read_byte(fd, &tmp8);
	PIA_1B.register_select = 0; pia_data_write(&PIA_1B, tmp8);  /* DDR */
	filer_read_byte(fd, &tmp8);
	PIA_1B.register_select = 1; pia_data_write(&PIA_1B, tmp8);  /* OR */
	filer_read_byte(fd, &tmp8);
	pia_control_write(&PIA_1B, tmp8);  /* CR */
	/* SAM */
	filer_read_word16(fd, &tmp16);
	sam_register = tmp16; sam_update_from_register();
	filer_close(fd);
	return 0;
}
