/*  XRoar - a Dragon/Tandy Coco emulator
 *  Copyright (C) 2003  Ciaran Anscomb
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include "xroar.h"
#include "joystick.h"
#include "keyboard.h"
#include "m6809.h"
#include "pia.h"
#include "sam.h"
#include "sound.h"
#include "vdg.h"
#include "filer.h"
#include "snapshot.h"
#include "tape.h"
#include "logging.h"

#ifdef TRACE
int trace = 0;
#endif

void xroar_init(void) {
	/* Initialise everything */
	filer_init();
	sam_init();
	/* pia_init() must be called before devices that connect to PIAs */
	pia_init();
	m6809_init();
	vdg_init();
	keyboard_init();
	joystick_init();
	sound_init();
	tape_init();
}

void xroar_reset(void) {
	/* Reset everything */
	keyboard_reset();
	joystick_reset();
	pia_reset();
	sam_ram_reset();
	sam_reset();
	m6809_reset();
	vdg_reset();
	sound_reset();
	tape_reset();
	xroar_dumpstate();
}

void xroar_mainloop(void) {
	int i;
	while (1) {
		for (i = 24; i; i--) {
			m6809_cycle();
			PIA_SET_P0CA1;
			keyboard_poll();
		}
		for (i = 192; i; i--) {
			vdg_cycle();
			m6809_cycle();
			PIA_SET_P0CA1;
			keyboard_poll();
		}
		for (i = 24; i; i--) {
			m6809_cycle();
			PIA_SET_P0CA1;
			keyboard_poll();
		}
		vdg_vsync();
		PIA_SET_P0CB1;
		for (i = 73; i; i--) {
			m6809_cycle();
			keyboard_poll();
		}
		if (snapshot_state) {
			switch(snapshot_state) {
				case 1: xroar_dumpstate(); write_snapshot(); break;
				case 2: read_snapshot(); xroar_dumpstate(); break;
				case 3: xroar_reset(); /*trace ^= 1;*/ break;
			}
			snapshot_state = 0;
		}
	}
}

void xroar_dumpstate(void) {
#ifndef GP32
#ifdef TRACE
	fprintf(stderr, "cc=%02x  a=%02x  b=%02x  dp=%02x  x=%04x  y=%04x  u=%04x  s=%04x  pc=%04x\n", reg_cc, reg_a, reg_b, reg_dp, reg_x, reg_y, reg_u, reg_s, reg_pc);
	fprintf(stderr, "Interrupt state: %s%s%s%s%s\n", nmi?"NMI":"", firq?"FIRQ":"", irq?"IRQ":"", wait_for_interrupt?"WAIT_FOR_INTERRUPT":"", skip_register_push?"SKIP_REGISTER_PUSH":"");
	fprintf(stderr, "SAM register: %04x\n", sam_register);
	fprintf(stderr, "PIA0A: cr=%02x ddr=%02x or=%02x tl=%02x th=%02x int=%s rs=%s\n", PIA_0A.control_register, PIA_0A.direction_register, PIA_0A.output_register, PIA_0A.tied_low, PIA_0A.tied_high, PIA_0A.interrupt_enable?"enable":"disable", PIA_0A.register_select?"OR":"DDR");
	fprintf(stderr, "PIA0B: cr=%02x ddr=%02x or=%02x tl=%02x th=%02x int=%s rs=%s\n", PIA_0B.control_register, PIA_0B.direction_register, PIA_0B.output_register, PIA_0B.tied_low, PIA_0B.tied_high, PIA_0B.interrupt_enable?"enable":"disable", PIA_0B.register_select?"OR":"DDR");
	fprintf(stderr, "PIA1A: cr=%02x ddr=%02x or=%02x tl=%02x th=%02x int=%s rs=%s\n", PIA_1A.control_register, PIA_1A.direction_register, PIA_1A.output_register, PIA_1A.tied_low, PIA_1A.tied_high, PIA_1A.interrupt_enable?"enable":"disable", PIA_1A.register_select?"OR":"DDR");
	fprintf(stderr, "PIA1B: cr=%02x ddr=%02x or=%02x tl=%02x th=%02x int=%s rs=%s\n", PIA_1B.control_register, PIA_1B.direction_register, PIA_1B.output_register, PIA_1B.tied_low, PIA_1B.tied_high, PIA_1B.interrupt_enable?"enable":"disable", PIA_1B.register_select?"OR":"DDR");
#endif
#endif
}
